@@ -4,7 +4,7 @@ module TeX
       class Package
         attr_reader :name
 
-        @@metadata = YAML::load File.read File.expand_path '../packages.yml', __dir__
+        @@metadata = Psych::safe_load File.read File.expand_path '../packages.yml', __dir__
 
         def initialize(name)
           @name = name
@@ -104,7 +104,7 @@ module TeX
           unless has_dependency?
             languages.each do |language|
               if language.use_old_patterns_comment and language.legacy_patterns != "zerohyph.tex" and language.bcp47 != 'cop'
-                if language.bcp47 == 'la-x-classic'
+                if ['la-x-classic', 'mk'].include? language.bcp47 # FIXME.  Yes, fix it ;-)
                   files << sprintf("tex/generic/hyph-utf8/patterns/tex-8bit/%s", language.legacy_patterns)
                 else
                   files << sprintf("tex/generic/hyphen/%s", language.legacy_patterns)
