@@ -1,6 +1,6 @@
 -- tkz_elements_functions_lines.lua
--- date 2024/04/27
--- version 2.25c
+-- date 2025/01/06
+-- version 3.10
 -- Copyright 2024  Alain Matthes
 -- This work may be distributed and/or modified under the
 -- conditions of the LaTeX Project Public License, either version 1.3
@@ -15,16 +15,17 @@
 ---------------------------------------------------------------------------
 --                 Lines
 ---------------------------------------------------------------------------
-function normalize_ (a,b)
-   return a+(b-a)/point.mod(b-a)
+function normalize_(a, b)
+    return a + (b - a) / point.mod(b - a)
 end
 
-function ortho_from_ ( p , a , b )
-	return  p+(b-a)*point(0,1) 
+
+function ortho_from_(p, a, b)
+    return p + (b - a) * point(0, 1)
 end
 
 function ll_from_ ( p , a , b )
-	return  p+b-a
+	return  p + b - a
 end
 
 function slope_ (a,b)
@@ -32,7 +33,7 @@ function slope_ (a,b)
 end
 
 function gold_segment_ (a,b)
-  return a + (b-a)*tkzinvphi
+  return a + (b - a) * tkzinvphi
 end
 
 function online_ (a,b,t)
@@ -52,26 +53,25 @@ function equilateral_tr_ (a,b)
    return rotation_ (a,math.pi/3,b)
 end
 
-function isosceles_right_tr (a,b)
-     local pt
-     pt = rotation_ (a,math.pi/4,b)
-   return a + (pt-a) * math.sin(math.pi/4)
+function isosceles_right_tr(a, b)
+    local pt = rotation_(a, math.pi / 4, b)
+    return a + (pt - a) * math.sin(math.pi / 4)
 end
 
-function gold_tr (a,b)
-   local pt
-   pt = rotation_ (a,math.pi/2,b)
-   return a + (pt-a) * tkzinvphi
+
+function gold_tr(a, b)
+    local pt = rotation_(a, math.pi / 2, b)
+    return a + (pt - a) * tkzinvphi
 end
 
+
 function euclide_tr (a,b)
      return rotation_ (a,math.pi/5,b)
 end
 
 function golden_tr (a,b)
-    local pt
-    pt = rotation_ (a,2*math.pi/5,b)
-     return a + (pt-a) * tkzphi
+   local  pt = rotation_ (a,2*math.pi/5,b)
+   return a + (pt-a) * tkzphi
 end
 
 function div_harmonic_int_(a,b,n)
@@ -97,66 +97,56 @@ function projection ( Dt,pt )
   return projection_ ( Dt.pa,Dt.pb,pt )
 end
 
-function projection_ ( pa,pb,pt )
-   local v
-   local z
-   if aligned ( pa,pb,pt ) then
-   return pt
-   else
-    v = pb - pa
-    z = ((pt - pa)..v)/(point.norm(v)) -- .. dot product
-   return pa + z * v  
-   end
+function projection_(pa, pb, pt)
+    if aligned(pa, pb, pt) then
+        return pt
+    else
+        local v = pb - pa
+        local z = ((pt - pa) .. v) / point.norm(v)  -- .. dot product
+        return pa + z * v
+    end
 end
 
+
 function symmetry_axial_(pa,pb,pt)
-    local p 
-    p = projection_ (pa,pb,pt)
+    local p = projection_ (pa,pb,pt)
     return symmetry_(p,pt)
 end
 
-function set_symmetry_axial_ (u,v,...)
-	local tp = table.pack(...)
-	local i
+function set_symmetry_axial_(u, v, ...)
     local t = {}
-	for i=1,tp.n do
-        table.insert( t , symmetry_axial_ (u,v , tp[i])  ) 
-	end
-  return table.unpack ( t )
+    for _, value in ipairs({...}) do
+        table.insert(t, symmetry_axial_(u, v, value))
+    end
+    return table.unpack(t)
 end
 
+
 function square_ (a,b)
     return rotation_ (b,-math.pi/2,a), rotation_ (a,math.pi/2,b)
 end
 
-function in_segment_ (a,b,pt)
-    local sc
-    sc = point.mod (pt-a) + point.mod (pt-b) - point.mod(b-a)
-    if sc <= tkz_epsilon
-     then
-       return true
+function in_segment_(a, b, pt)
+    return point.mod(pt - a) + point.mod(pt - b) - point.mod(b - a) <= tkz_epsilon
+end
+
+function report_(za, zb, d, pt)
+    local len = point.mod(zb - za)
+    local t = d / len
+    local result = barycenter_({za, 1 - t}, {zb, t})
+    
+    if pt then
+        return result + pt - za
     else
-       return false
+        return result
     end
 end
 
-function report_ (za,zb,d,pt)
-   local t,len
-   len = point.mod(zb-za)
-   t = d/len
-   if pt == nil  
-   then  
-   return barycenter_({za,1-t},{zb,(t)})
-else 
-   return barycenter_({za,1-t},{zb,(t)}) +pt-za
-end
+function colinear_at_(za, zb, pt, k)
+    if k then
+        return pt + k * (zb - za)
+    else
+        return pt + (zb - za)
+    end
 end
 
-function colinear_at_ (za,zb,pt,k)
-  if k == nil  
-  then 
-	return  pt+zb-za
-else
-  return  pt+k*(zb-za)
-end
-end
