@@ -1,6 +1,6 @@
 -- tkz_elements_point.lua
--- date 2024/04/27
--- version 2.25c
+-- date 2025/01/06
+-- version 3.10
 -- Copyright 2024  Alain Matthes
 -- This work may be distributed and/or modified under the
 -- conditions of the LaTeX Project Public License, either version 1.3
@@ -252,28 +252,28 @@ function point: west(d)
 --     return symmetry_ (self ,pt)
 -- end
 
-function point: symmetry (...)
-   local obj,nb,t
-   local tp = table.pack(...)
-   obj = tp[1]
-   nb = tp.n
-    if nb == 1 then
-       if obj.type == "point" then
-          return symmetry_ (self,obj)
-       elseif  obj.type == "line" then
-         return line: new (set_symmetry_ (self,obj.pa,obj.pb))
-       elseif  obj.type == "circle" then
-          return circle: new (set_symmetry_ (self,obj.center,obj.through))
-       else
-         return triangle: new (set_symmetry (self,obj.pa,obj.pb,obj.pc)) 
-       end
-    else
-       local t = {}
-       for i=1,tp.n do
-           table.insert( t , symmetry_ (self , tp[i])  ) 
-        end
-     return table.unpack ( t )      
-    end
+function point:symmetry(...)
+   local tp = table.pack(...)  -- Pack arguments into a table
+   local nb = tp.n  -- Number of arguments
+   local obj = tp[1]  -- The first object in the arguments
+
+   if nb == 1 then  -- If there's only one argument
+      if obj.type == "point" then
+         return symmetry_(self, obj)  -- Apply symmetry on the point
+      elseif obj.type == "line" then
+         return line:new(set_symmetry_(self, obj.pa, obj.pb))  -- Create a new line
+      elseif obj.type == "circle" then
+         return circle:new(set_symmetry_(self, obj.center, obj.through))  -- Create a new circle
+      else
+         return triangle:new(set_symmetry(self, obj.pa, obj.pb, obj.pc))  -- Create a new triangle
+      end
+   else  -- If there are multiple arguments
+      local results = {}  -- Initialize a table to store results
+      for i = 1, nb do
+         table.insert(results, symmetry_(self, tp[i]))  -- Apply symmetry on each object
+      end
+      return table.unpack(results)  -- Return the results as separate values
+   end
 end
 
 function point: set_symmetry (...)
@@ -288,56 +288,56 @@ function point:set_rotation (angle,...)
  return set_rotation_ ( self,angle,... )
 end
 
-function point : rotation (angle,...)
-   local obj,nb,t
-   local tp = table.pack(...)
-   obj = tp[1]
-   nb = tp.n
-    if nb == 1 then
-       if obj.type == "point" then
-      return rotation_ (self,angle,obj )
-       elseif  obj.type == "line" then
-   return line : new  (set_rotation_ (self, angle,obj.pa,obj.pb ))
-       elseif obj.type == "triangle" then
-   return triangle: new (set_rotation_ (self, angle,obj.pa,obj.pb,obj.pc))
-       elseif obj.type == "circle" then
-return circle : new  (set_rotation_ (self,angle,obj.center,obj.through))
-       else 
-return square: new (set_rotation_(self,angle,obj.pa,obj.pb,obj.pc,obj.pd))
-       end
-    else
-        t = {}
-        for i=1,tp.n do
-            table.insert( t , rotation_ ( self,angle,tp[i]))
-         end
-      return table.unpack ( t )
-     end
+function point:rotation(angle, ...)
+   local tp = table.pack(...)  -- Pack arguments into a table
+   local nb = tp.n  -- Number of arguments
+   local obj = tp[1]  -- The first object in the arguments
+
+   if nb == 1 then  -- If there's only one argument
+      if obj.type == "point" then
+         return rotation_(self, angle, obj)  -- Rotate the point
+      elseif obj.type == "line" then
+         return line:new(set_rotation_(self, angle, obj.pa, obj.pb))  -- Rotate the line
+      elseif obj.type == "triangle" then
+         return triangle:new(set_rotation_(self, angle, obj.pa, obj.pb, obj.pc))  -- Rotate the triangle
+      elseif obj.type == "circle" then
+         return circle:new(set_rotation_(self, angle, obj.center, obj.through))  -- Rotate the circle
+      else  -- For other shapes like square
+         return square:new(set_rotation_(self, angle, obj.pa, obj.pb, obj.pc, obj.pd))  -- Rotate the square
+      end
+   else  -- If there are multiple arguments
+      local results = {}  -- Initialize a table to store results
+      for i = 1, nb do
+         table.insert(results, rotation_(self, angle, tp[i]))  -- Rotate each object
+      end
+      return table.unpack(results)  -- Return the results as separate values
+   end
 end
 
-function point : homothety (coeff,...)
-local obj,nb,t
-local tp = table.pack(...)
-obj = tp[1]
-nb = tp.n
- if nb == 1 then
-   if obj.type == "point" then
-      return homothety_ (self,coeff,obj )
-   elseif  obj.type == "line" then
-      return line : new  (set_homothety_ (self, coeff,obj.pa,obj.pb ))
-   elseif obj.type == "triangle" then
-     return triangle: new (set_homothety_(self,coeff,obj.pa,obj.pb,obj.pc))
-   elseif obj.type == "circle" then
-    return circle: new  (set_homothety_(self,coeff,obj.center,obj.through))
-   else
-return square: new (set_homothety_(self,coeff,obj.pa,obj.pb))    
-       end
-    else
-        t = {}
-        for i=1,tp.n do
-            table.insert( t , homothety_ ( self,coeff,tp[i]))
-         end
-      return table.unpack ( t )
-     end
+function point:homothety(coeff, ...)
+    local tp = table.pack(...)  -- Pack arguments into a table
+    local nb = tp.n  -- Number of arguments
+    local obj = tp[1]  -- The first object in the arguments
+    local t = {}  -- Initialize a table to store results
+
+    if nb == 1 then  -- If there's only one argument
+        if obj.type == "point" then
+            return homothety_(self, coeff, obj)  -- Apply homothety to the point
+        elseif obj.type == "line" then
+            return line:new(set_homothety_(self, coeff, obj.pa, obj.pb))  -- Apply homothety to the line
+        elseif obj.type == "triangle" then
+            return triangle:new(set_homothety_(self, coeff, obj.pa, obj.pb, obj.pc))  -- Apply homothety to the triangle
+        elseif obj.type == "circle" then
+            return circle:new(set_homothety_(self, coeff, obj.center, obj.through))  -- Apply homothety to the circle
+        else  -- For other shapes like square
+            return square:new(set_homothety_(self, coeff, obj.pa, obj.pb, obj.pc, obj.pd))  -- Apply homothety to the square
+        end
+    else  -- If there are multiple arguments
+        for i = 1, nb do
+            table.insert(t, homothety_(self, coeff, tp[i]))  -- Apply homothety to each object
+        end
+        return table.unpack(t)  -- Return the results as separate values
+    end
 end
 
 function point: normalize()
@@ -345,16 +345,19 @@ function point: normalize()
    return point(self.re/d,self.im/d)
 end
 
-function point: orthogonal(d)
+function point:orthogonal(d)
    local m
-   if d==nil then
-   return point(-self.im,self.re)
-else
-   m = point.mod(self)
-   return point(-self.im*d/m,self.re*d/m)
-end
+   if d == nil then
+      -- If no scaling factor d is provided, return the point rotated 90 degrees counterclockwise
+      return point(-self.im, self.re)
+   else
+      -- If a scaling factor d is provided, scale the orthogonal point
+      m = point.mod(self)  -- Get the modulus (magnitude) of the current point
+      return point(-self.im * d / m, self.re * d / m)  -- Return the scaled orthogonal point
+   end
 end
 
+
 function point : at (z)
    return point(self.re+z.re,self.im+z.im)
 end
